@@ -335,9 +335,7 @@ public:
 
     ~LogStreamPrivate()
     {
-        for (simgear::LogCallback* cb : m_callbacks) {
-            delete cb;
-        }
+        removeCallbacks();
     }
 
     SGMutex m_lock;
@@ -363,6 +361,9 @@ public:
 #endif
     bool m_developerMode = false;
 
+    // test suite mode.
+    bool m_testMode = false;
+
     void startLog()
     {
         SGGuard<SGMutex> g(m_lock);
@@ -443,6 +444,16 @@ public:
         }
     }
 
+    void removeCallbacks()
+    {
+        PauseThread pause(this);
+        for (simgear::LogCallback* cb : m_callbacks) {
+            delete cb;
+        }
+        m_callbacks.clear();
+        m_consoleCallbacks.clear();
+    }
+
     void setLogLevels( sgDebugClass c, sgDebugPriority p )
     {
         PauseThread pause(this);
@@ -455,6 +466,9 @@ public:
 
     bool would_log( sgDebugClass c, sgDebugPriority p ) const
     {
+        // Testing mode, so always log.
+        if (m_testMode) return true;
+
         p = translatePriority(p);
         if (p >= SG_INFO) return true;
         return ((c & m_logClass) != 0 && p >= m_logPriority);
@@ -711,6 +725,13 @@ void logstream::requestConsole()
 #endif
 }
 
+void
+logstream::setTestingMode( bool testMode )
+{
+    d->m_testMode = testMode;
+    if (testMode) d->removeCallbacks();
+}
+
 
 namespace simgear
 {
